"""
Definitions of BF and Weight dicts for the D BF Reweighting

Patrick Ecker 2020
"""

from rdstar.mc_reweighting.BFReweighting.d_reweighting_utils import (
    DMesonBFInfo,
    BFTupleType,
    BFDictType,
    PDGBFDictType,
    generate_bf_dict,
    generate_bf_weight_dict,
    generate_charge_conjugate_bf_weight_dict,
)

__all__ = [
    "d_charged_bf_weight_dict",
    "d_zero_bf_weight_dict",
    "anti_d_charged_bf_weight_dict",
    "anti_d_zero_bf_weight_dict",
]


# D+ Decays
# fmt: off
belle_d_charged_bf = (
    DMesonBFInfo(decay_products="anti-K*0   e+  nu_e", bf=0.0310),
    DMesonBFInfo(decay_products="anti-K0    e+  nu_e", bf=0.0680),
    DMesonBFInfo(decay_products="anti-K_10  e+  nu_e", bf=0.0200),
    DMesonBFInfo(decay_products="anti-K_2*0 e+  nu_e", bf=0.0090),
    DMesonBFInfo(decay_products="pi0   e+  nu_e", bf=0.0031),
    DMesonBFInfo(decay_products="eta   e+  nu_e", bf=0.0030),
    DMesonBFInfo(decay_products="eta'  e+  nu_e", bf=0.0005),
    DMesonBFInfo(decay_products="rho0  e+  nu_e", bf=0.0031),
    DMesonBFInfo(decay_products="omega e+  nu_e", bf=0.0031),
    DMesonBFInfo(decay_products="b_10  e+  nu_e", bf=0.0002),
    DMesonBFInfo(decay_products="K-       pi+  e+ nu_e", bf=0.0100),
    DMesonBFInfo(decay_products="anti-K0  pi0  e+ nu_e", bf=0.0100),
    DMesonBFInfo(decay_products="anti-K*0   mu+  nu_mu", bf=0.0310),
    DMesonBFInfo(decay_products="anti-K0    mu+  nu_mu", bf=0.0680),
    DMesonBFInfo(decay_products="anti-K_10  mu+  nu_mu", bf=0.0200),
    DMesonBFInfo(decay_products="anti-K_2*0 mu+  nu_mu", bf=0.0090),
    DMesonBFInfo(decay_products="pi0   mu+  nu_mu", bf=0.0031),
    DMesonBFInfo(decay_products="eta   mu+  nu_mu", bf=0.0030),
    DMesonBFInfo(decay_products="eta'  mu+  nu_mu", bf=0.0005),
    DMesonBFInfo(decay_products="rho0  mu+  nu_mu", bf=0.0031),
    DMesonBFInfo(decay_products="omega mu+  nu_mu", bf=0.0031),
    DMesonBFInfo(decay_products="b_10  mu+  nu_mu", bf=0.0002),
    DMesonBFInfo(decay_products="K-       pi+  mu+ nu_mu", bf=0.0100),
    DMesonBFInfo(decay_products="anti-K0  pi0  mu+ nu_mu", bf=0.0100),
    DMesonBFInfo(decay_products="mu+  nu_mu", bf=0.0008),
    DMesonBFInfo(decay_products="tau+ nu_tau", bf=0.0024),
    DMesonBFInfo(decay_products="anti-K0   pi+", bf=0.0294),
    DMesonBFInfo(decay_products="rho+ anti-K0", bf=0.0640),
    DMesonBFInfo(decay_products="a_1+ anti-K0", bf=0.0695),
    DMesonBFInfo(decay_products="anti-K*0    pi+", bf=0.0195),
    DMesonBFInfo(decay_products="anti-K'_10  pi+", bf=0.0380),
    DMesonBFInfo(decay_products="anti-K_0*0  pi+", bf=0.0378),
    DMesonBFInfo(decay_products="anti-K''*0  pi+", bf=0.0147),
    DMesonBFInfo(decay_products="anti-K*0 rho+", bf=0.0081),
    DMesonBFInfo(decay_products="K-  pi+  pi+", bf=0.0531),
    DMesonBFInfo(decay_products="anti-K0   pi+  pi0", bf=0.0451),
    DMesonBFInfo(decay_products="anti-K0   eta  pi+", bf=0.0100),
    DMesonBFInfo(decay_products="anti-K0   rho0  pi+", bf=0.0013),
    DMesonBFInfo(decay_products="anti-K0   omega  pi+", bf=0.0020),
    DMesonBFInfo(decay_products="K-        rho+  pi+", bf=0.0073),
    DMesonBFInfo(decay_products="K*-      pi+   pi+", bf=0.0126),
    DMesonBFInfo(decay_products="anti-K*0 pi0   pi+", bf=0.0301),
    DMesonBFInfo(decay_products="anti-K*0 eta   pi+", bf=0.0050),
    DMesonBFInfo(decay_products="anti-K*0 rho0  pi+", bf=0.0030),
    DMesonBFInfo(decay_products="anti-K*0 omega pi+", bf=0.0020),
    DMesonBFInfo(decay_products="K*-      rho+  pi+", bf=0.0100),
    DMesonBFInfo(decay_products="K-        pi+  pi+   pi0", bf=0.0014),
    DMesonBFInfo(decay_products="anti-K0   pi+  pi0   pi0", bf=0.0075),
    DMesonBFInfo(decay_products="K-        rho0 pi+   pi+", bf=0.0019),
    DMesonBFInfo(decay_products="anti-K*0  pi+  pi+   pi-", bf=0.0029),
    DMesonBFInfo(decay_products="K-        pi+  pi+   pi+   pi-", bf=0.0004),
    DMesonBFInfo(decay_products="K-        pi+  pi+   pi0   pi0", bf=0.0117),
    DMesonBFInfo(decay_products="anti-K0   pi+  pi+   pi-   pi0", bf=0.0118),
    DMesonBFInfo(decay_products="anti-K0   pi+  pi0   pi0   pi0", bf=0.0040),
    DMesonBFInfo(decay_products="anti-K0   pi+  pi+   pi+   pi-  pi-", bf=0.0010),
    DMesonBFInfo(decay_products="anti-K0   anti-K0    K+", bf=0.0180),
    DMesonBFInfo(decay_products="anti-K0   K+  K- pi+", bf=0.0006),
    DMesonBFInfo(decay_products="phi pi+", bf=0.0057),
    DMesonBFInfo(decay_products="phi pi+  pi0", bf=0.0230),
    DMesonBFInfo(decay_products="anti-K0   K+", bf=0.0059),
    DMesonBFInfo(decay_products="anti-K*0  K+", bf=0.0038),
    DMesonBFInfo(decay_products="K*+  anti-K0", bf=0.0310),
    DMesonBFInfo(decay_products="anti-K*0 K*+", bf=0.0260),
    DMesonBFInfo(decay_products="K+   K-  pi+", bf=0.0037),
    DMesonBFInfo(decay_products="K+  anti-K0   pi0", bf=0.0010),
    DMesonBFInfo(decay_products="anti-K0  K0   pi+", bf=0.0010),
    DMesonBFInfo(decay_products="K*+ K-  pi+", bf=0.0010),
    DMesonBFInfo(decay_products="K+  K*- pi+", bf=0.0030),
    DMesonBFInfo(decay_products="K*+ anti-K0   pi0", bf=0.0010),
    DMesonBFInfo(decay_products="K+  anti-K*0  pi0", bf=0.0010),
    DMesonBFInfo(decay_products="anti-K*0 K0   pi+", bf=0.0010),
    DMesonBFInfo(decay_products="anti-K0  K*0  pi+", bf=0.0030),
    DMesonBFInfo(decay_products="pi0 pi+", bf=0.0026),
    DMesonBFInfo(decay_products="rho0 pi+", bf=0.0011),
    DMesonBFInfo(decay_products="pi+  pi+  pi-", bf=0.0020),
    DMesonBFInfo(decay_products="pi+  pi0  pi0", bf=0.0031),
    DMesonBFInfo(decay_products="pi+  pi+  pi-  pi0", bf=0.0100),
    DMesonBFInfo(decay_products="pi+  pi0  pi0  pi0", bf=0.0020),
    DMesonBFInfo(decay_products="pi+  pi+  pi+  pi-  pi-", bf=0.0020),
    DMesonBFInfo(decay_products="pi+  pi+  pi+  pi-  pi-  pi0", bf=0.0020),
    DMesonBFInfo(decay_products="eta  pi+", bf=0.0030),
    DMesonBFInfo(decay_products="eta' pi+", bf=0.0051),
    DMesonBFInfo(decay_products="f_2 pi+", bf=0.0011),
    DMesonBFInfo(decay_products="eta  pi+ pi0", bf=0.0030),
    DMesonBFInfo(decay_products="eta  pi+ pi+ pi-", bf=0.0010),
    DMesonBFInfo(decay_products="eta  pi+ pi0 pi0", bf=0.0010),
    DMesonBFInfo(decay_products="K*0 pi+", bf=0.0004),
    DMesonBFInfo(decay_products="rho0 K+", bf=0.0003),
    DMesonBFInfo(decay_products="K+ pi+ pi-", bf=0.0002),
    DMesonBFInfo(decay_products="K+ K+ K-", bf=0.0001),
)  # type: BFTupleType
# fmt: on

# D0 Decays
# fmt: off
belle_d_zero_bf = (
    DMesonBFInfo(decay_products="K-   e+  nu_e", bf=0.0341),
    DMesonBFInfo(decay_products="K*-  e+  nu_e", bf=0.0207),
    DMesonBFInfo(decay_products="K_2*- e+  nu_e", bf=0.0018),
    DMesonBFInfo(decay_products="pi-  e+  nu_e", bf=0.0026),
    DMesonBFInfo(decay_products="rho- e+  nu_e", bf=0.0019),
    DMesonBFInfo(decay_products="b_1- e+  nu_e", bf=0.0005),
    DMesonBFInfo(decay_products="K-  pi0  e+ nu_e", bf=0.0022),
    DMesonBFInfo(decay_products="anti-K0  pi-  e+ nu_e", bf=0.0022),
    DMesonBFInfo(decay_products="K-   mu+ nu_mu", bf=0.0341),
    DMesonBFInfo(decay_products="K*-  mu+ nu_mu", bf=0.0207),
    DMesonBFInfo(decay_products="K_2*- mu+ nu_mu", bf=0.0018),
    DMesonBFInfo(decay_products="pi-  mu+ nu_mu", bf=0.0026),
    DMesonBFInfo(decay_products="rho- mu+ nu_mu", bf=0.0019),
    DMesonBFInfo(decay_products="b_1- mu+  nu_mu", bf=0.0005),
    DMesonBFInfo(decay_products="K-  pi0  mu+ nu_mu", bf=0.0022),
    DMesonBFInfo(decay_products="anti-K0  pi-  mu+ nu_mu", bf=0.0022),
    DMesonBFInfo(decay_products="K-  pi+", bf=0.0382),
    DMesonBFInfo(decay_products="anti-K0   pi0", bf=0.0226),
    DMesonBFInfo(decay_products="anti-K0   eta", bf=0.0076),
    DMesonBFInfo(decay_products="anti-K0   eta'", bf=0.0188),
    DMesonBFInfo(decay_products="rho0 anti-K0", bf=0.0102),
    DMesonBFInfo(decay_products="rho+ K-", bf=0.0896),
    DMesonBFInfo(decay_products="omega anti-K0", bf=0.0218),
    DMesonBFInfo(decay_products="anti-K*0 eta", bf=0.0180),
    DMesonBFInfo(decay_products="a_1+ K-", bf=0.0750),
    DMesonBFInfo(decay_products="K*- rho+", bf=0.0640),
    DMesonBFInfo(decay_products="anti-K*0 rho0", bf=0.0110),
    DMesonBFInfo(decay_products="anti-K*0 omega", bf=0.0110),
    DMesonBFInfo(decay_products="anti-K*0 pi0", bf=0.0198),
    DMesonBFInfo(decay_products="K*-  pi+", bf=0.0517),
    DMesonBFInfo(decay_products="K_1- pi+", bf=0.0110),
    DMesonBFInfo(decay_products="f_0   anti-K0", bf=0.0120),
    DMesonBFInfo(decay_products="f'_0  anti-K0", bf=0.0120),
    DMesonBFInfo(decay_products="K_0*-  pi+", bf=0.0058),
    DMesonBFInfo(decay_products="K_2*-  pi+", bf=0.0004),
    DMesonBFInfo(decay_products="anti-K_0*0  pi0", bf=0.0034),
    DMesonBFInfo(decay_products="K-  pi+  pi0", bf=0.0066),
    DMesonBFInfo(decay_products="anti-K0   pi0  pi0", bf=0.0070),
    DMesonBFInfo(decay_products="anti-K*0 pi+  pi-", bf=0.0106),
    DMesonBFInfo(decay_products="anti-K*0 pi0  pi0", bf=0.0106),
    DMesonBFInfo(decay_products="K-  rho+ pi0", bf=0.0068),
    DMesonBFInfo(decay_products="K-  pi+  rho0", bf=0.0046),
    DMesonBFInfo(decay_products="K-  pi+  omega", bf=0.0226),
    DMesonBFInfo(decay_products="K-  pi+  eta", bf=0.0070),
    DMesonBFInfo(decay_products="K-  pi+  eta'", bf=0.0070),
    DMesonBFInfo(decay_products="anti-K*0 pi+  pi-   pi0", bf=0.0041),
    DMesonBFInfo(decay_products="K-  pi+  pi+   pi-", bf=0.0117),
    DMesonBFInfo(decay_products="anti-K0   pi+  pi-   pi0", bf=0.0256),
    DMesonBFInfo(decay_products="K-  pi+  pi0   pi0", bf=0.0400),
    DMesonBFInfo(decay_products="anti-K0   pi0  pi0   pi0", bf=0.0165),
    DMesonBFInfo(decay_products="K-  pi+  pi+   pi-   pi0", bf=0.0049),
    DMesonBFInfo(decay_products="K-  pi+  pi0   pi0   pi0", bf=0.0050),
    DMesonBFInfo(decay_products="anti-K0   pi+  pi-   pi0   pi0", bf=0.0260),
    DMesonBFInfo(decay_products="anti-K0   pi+  pi-   pi0   pi0   pi0", bf=0.0268),
    DMesonBFInfo(decay_products="phi anti-K0", bf=0.0094),
    DMesonBFInfo(decay_products="phi anti-K*0", bf=0.0003),
    DMesonBFInfo(decay_products="phi K- pi+", bf=0.00007),
    DMesonBFInfo(decay_products="anti-K*0 K+ K-", bf=0.00006),
    DMesonBFInfo(decay_products="anti-K0   K+   K-", bf=0.0048),
    DMesonBFInfo(decay_products="K_S0 K_S0  K_S0", bf=0.0009),
    DMesonBFInfo(decay_products="K_L0 K_L0  K_L0", bf=0.0009),
    DMesonBFInfo(decay_products="K+  K-", bf=0.0039),
    DMesonBFInfo(decay_products="K_S0 K_S0", bf=0.0004),
    DMesonBFInfo(decay_products="K_L0 K_L0", bf=0.0004),
    DMesonBFInfo(decay_products="K*- K+", bf=0.0012),
    DMesonBFInfo(decay_products="K*+ K-", bf=0.0044),
    DMesonBFInfo(decay_products="anti-K*0 K*0", bf=0.0002),
    DMesonBFInfo(decay_products="phi pi0", bf=0.0010),
    DMesonBFInfo(decay_products="phi eta", bf=0.00014),
    DMesonBFInfo(decay_products="K0    K-   pi+", bf=0.0064),
    DMesonBFInfo(decay_products="anti-K0    K+   pi-", bf=0.0052),
    DMesonBFInfo(decay_products="phi  rho0", bf=0.0013),
    DMesonBFInfo(decay_products="phi pi+  pi-", bf=0.0005),
    DMesonBFInfo(decay_products="K+  K-   rho0", bf=0.0009),
    DMesonBFInfo(decay_products="K+  K-   pi0  pi0", bf=0.0030),
    DMesonBFInfo(decay_products="K*0   K-   pi+", bf=0.0003),
    DMesonBFInfo(decay_products="anti-K*0   K+   pi-", bf=0.0002),
    DMesonBFInfo(decay_products="anti-K0   K0   pi+  pi-", bf=0.0068),
    DMesonBFInfo(decay_products="K+    K-    K-    pi+", bf=0.00023),
    # this decay is defined multiple times in the dec file -> bf = sum of all definitions
    DMesonBFInfo(decay_products="K+    K-    anti-K0    pi0", bf=0.0072),
    DMesonBFInfo(decay_products="K+    K-    pi+   pi-   pi0", bf=0.0031),
    DMesonBFInfo(decay_products="pi+ pi-", bf=0.0014),
    DMesonBFInfo(decay_products="pi0 pi0", bf=0.0008),
    DMesonBFInfo(decay_products="eta pi0", bf=0.0006),
    DMesonBFInfo(decay_products="eta'  pi0", bf=0.0006),
    DMesonBFInfo(decay_products="eta eta", bf=0.0006),
    DMesonBFInfo(decay_products="rho+ pi-", bf=0.0087),
    DMesonBFInfo(decay_products="rho- pi+", bf=0.0034),
    DMesonBFInfo(decay_products="rho0 pi0", bf=0.0019),
    DMesonBFInfo(decay_products="pi0  pi0  pi0", bf=0.0006),
    DMesonBFInfo(decay_products="pi+  pi+  pi-  pi-", bf=0.0073),
    DMesonBFInfo(decay_products="pi+  pi-  pi0  pi0", bf=0.0050),
    DMesonBFInfo(decay_products="pi+  pi-  pi+  pi-  pi0", bf=0.0190),
    DMesonBFInfo(decay_products="pi+  pi-  pi0  pi0  pi0", bf=0.0060),
    DMesonBFInfo(decay_products="pi+  pi-  pi+  pi-  pi+  pi-", bf=0.0004),
    DMesonBFInfo(decay_products="pi-  K+", bf=0.00014),
    DMesonBFInfo(decay_products="K*+  pi-", bf=0.0003),
    DMesonBFInfo(decay_products="pi-  K+  pi0", bf=0.00046),
    DMesonBFInfo(decay_products="pi-  K+  pi+   pi-", bf=0.0003),
)  # type: BFTupleType
# fmt: on

# Define the bf dicts
belle_d_charged_bf_dict = generate_bf_dict(bf_tuple=belle_d_charged_bf)
belle_d_zero_bf_dict = generate_bf_dict(bf_tuple=belle_d_zero_bf)

# Updated bf dicts
updated_d_charged_bf_dict = {
    ("anti-K*0", "e+", "nu_e"): 0.054,  # PDG 25
    ("anti-K0", "e+", "nu_e"): 0.0873,  # PDG 16
    ("anti-K_10", "e+", "nu_e"): 0.00106,  # PDG 31
    ("K-", "pi+", "e+", "nu_e"): 0.0402 - 0.0377 - 0.005,  # PDG 18 - 19 - 23
    ("anti-K0", "pi0", "e+", "nu_e"): 0.00,  # not found
    ("anti-K*0", "mu+", "nu_mu"): 0.0527,  # PDG 29
    ("anti-K0", "mu+", "nu_mu"): 0.0876,  # PDG 17
    ("anti-K_10", "mu+", "nu_mu"): 0.0106,  # similar to PDG 31
    ("K-", "pi+", "mu+", "nu_mu"): 0.0365 - 0.0352 - 0.005,  # PDG 26 - 27 - 23
    ("anti-K0", "pi0", "mu+", "nu_mu"): 0.00,  # not found
    ("anti-K0", "pi+"): 0.01562 * 2,  # PDG 46 via K_S0
    ("rho+", "anti-K0"): 0.064,  # Checked with Pablo maybe 59
    ("a_1+", "anti-K0"): 0.0695,  # Checked with Pablo maybe 59
    ("anti-K*0", "pi+"): 0.0104,  # PDG 52
    ("anti-K'_10", "pi+"): 0.038,  # Checked with Pablo
    ("anti-K_0*0", "pi+"): 0.0125 + 0.0027,  # PDG 51 + 62
    ("anti-K''*0", "pi+"): 0.001,  # PDG 63
    ("K-", "pi+", "pi+"): 0.0938 - 0.0104 - 0.0125,  # PDG 48 - 52 - 51
    ("anti-K0", "pi+", "pi0"): 2 * (0.0736 - 0.0614 - 0.0027 - 0.001),  # PDG 58 - 61 - 62 - 63
    ("anti-K0", "eta", "pi+"): 0.00,  # not found
    ("K*-", "pi+", "pi+"): 0.009274,  # Belle 2 DecFile Oct 2020
    ("anti-K*0", "pi0", "pi+"): 0.047188,  # Belle 2 DecFile Oct 2020
    ("K*-", "rho+", "pi+"): 0.00,  # not found
    ("K-", "pi+", "pi+", "pi0", "pi0"): 0.003851,  # Belle 2 DecFile Oct 2020
    ("anti-K0", "pi+", "pi+", "pi-", "pi0"): 0.006701,  # Belle 2 DecFile Oct 2020
    ("anti-K0", "anti-K0", "K+"): 4 * 0.00254,  # PDG 78 via K_S0
    ("phi", "pi+", "pi0"): 0.023,  # PDG 120
    ("K*+", "anti-K0"): 2 * 0.017,  # PDG 123 via K_S0
    ("anti-K*0", "K*+"): 0.00,  # not found
    ("pi+", "pi+", "pi-", "pi0"): 0.0116,  # PDG 95
    # BF < 0.01
    ("anti-K_2*0", "e+", "nu_e"): 0.005,  # PDG 23
    ("anti-K_2*0", "mu+", "nu_mu"): 0.005,  # similar to PDG 23
    ("anti-K*0", "rho+"): 0.0093,  # similar to PDG 74
    ("K-", "rho+", "pi+"): 0.007033,  # Belle 2 DecFile Oct 2020
    ("anti-K*0", "eta", "pi+"): 0.00,  # not found
    ("anti-K0", "pi+", "pi0", "pi0"): 0.00,  # not found
    ("phi", "pi+"): 0.0057 - 0.00249,  # PDG 108 - 110
    ("anti-K0", "K+"): 2 * 0.00304,  # PDG 102 via K_S0
    ("eta'", "pi+"): 0.00497,  # PDG 100
    # BF < 0.005
    ("pi0", "e+", "nu_e"): 0.00372,  # PDG 34
    ("eta", "e+", "nu_e"): 0.00111,  # PDG 36
    ("rho0", "e+", "nu_e"): 0.00218,  # PDG 39
    ("omega", "e+", "nu_e"): 0.00169,  # PDG 41
    ("pi0", "mu+", "nu_mu"): 0.0035,  # PDG 35
    ("eta", "mu+", "nu_mu"): 0.00111,  # similar to PDG 36
    ("rho0", "mu+", "nu_mu"): 0.0024,  # PDG 40
    ("omega", "mu+", "nu_mu"): 0.00169,  # similar to PDG 41
    ("tau+", "nu_tau"): 0.0012,  # PDG 15
    ("anti-K0", "omega", "pi+"): 0.003851,  # Belle 2 DecFile Oct 2020
    ("anti-K*0", "rho0", "pi+"): 0.0023,  # PDG 73
    ("anti-K*0", "omega", "pi+"): 0.003851,  # Belle 2 DecFile Oct 2020
    ("anti-K*0", "pi+", "pi+", "pi-"): 0.0012,  # PDG 72
    ("anti-K0", "pi+", "pi0", "pi0", "pi0"): 0.002696,  # Belle 2 DecFile Oct 2020
    ("anti-K*0", "K+"): 0.00249,  # PDG 110
    ("K+", "K-", "pi+"): 0.00993,  # Belle 2 DecFile Oct 2020 Dalitz
    ("K+", "K*-", "pi+"): 0.00077,  # Belle 2 DecFile Oct 2020
    ("anti-K0", "K*0", "pi+"): 0.00,  # not found
    ("pi0", "pi+"): 0.001247,  # PDG 80
    ("pi+", "pi+", "pi-"): 0.00327,  # PDG 81
    ("pi+", "pi0", "pi0"): 0.0047,  # PDG 94
    ("pi+", "pi0", "pi0", "pi0"): 0.003851,  # Belle 2 DecFile Oct 2020
    ("pi+", "pi+", "pi+", "pi-", "pi-"): 0.00166,  # PDG 96
    ("pi+", "pi+", "pi+", "pi-", "pi-", "pi0"): 0.00,  # not found
    ("eta", "pi+"): 0.00377,  # PDG 97
    ("eta", "pi+", "pi0"): 0.00138,  # PDG 98
}  # type: BFDictType

updated_d_zero_bf_dict = {
    ("K-", "e+", "nu_e"): 0.03542,  # PDG 19
    ("K*-", "e+", "nu_e"): 0.0215,  # PDG 21
    ("pi-", "e+", "nu_e"): 0.00291,  # PDG 30
    ("K-", "pi0", "e+", "nu_e"): 0.00,  # PDG 23 since there are large discrepancies between PDG, Belle (2) set to 0
    ("anti-K0", "pi-", "e+", "nu_e"): 0.0144,  # PDG 24
    ("K-", "mu+", "nu_mu"): 0.0341,  # PDG 20
    ("K*-", "mu+", "nu_mu"): 0.0189,  # PDG 22
    ("pi-", "mu+", "nu_mu"): 0.00267,  # PDG 31
    ("K-", "pi0", "mu+", "nu_mu"): 0.00,
    # similar to PDG 23 since there are large discrepancies between PDG, Belle (2) set to 0
    ("anti-K0", "pi-", "mu+", "nu_mu"): 0.0144,  # similar to PDG 24
    ("K-", "pi+"): 0.0395,  # PDG 35
    ("anti-K0", "pi0"): 2 * 0.0124,  # PDG 36 via K_S0
    ("anti-K0", "eta"): 2 * 0.00509,  # PDG 99 via K_S0
    ("anti-K0", "eta'"): 2 * 0.00949,  # PDG 101 via K_S0
    ("rho0", "anti-K0"): 2 * 0.0063,  # PDG 39 via K_S0
    ("rho+", "K-"): 0.113,  # PDG 54
    ("omega", "anti-K0"): 2 * 0.0111,  # PDG 100 via K_S0
    ("anti-K*0", "eta"): 0.019,  # PDG 102 best estimate
    ("a_1+", "K-"): 0.0433,  # PDG 75
    ("K*-", "rho+"): 0.067909,  # Belle 2 DecFile Oct 2020
    ("anti-K*0", "rho0"): 0.0101,  # PDG 73
    ("anti-K*0", "omega"): 0.011,  # PDG 105
    ("anti-K*0", "pi0"): 0.0195,  # PDG 57
    ("K*-", "pi+"): 0.0164 + 0.0231,  # PDG 45 + 56
    ("K_1-", "pi+"): 0.0039,  # PDG 76
    ("f_0", "anti-K0"): 0.0012,  # not found in PDG or BELLE 2, therefore weighted down by one order of magnitude
    ("f'_0", "anti-K0"): 0.0012,  # not found in PDG or BELLE 2, therefore weighted down by one order of magnitude
    ("K_0*-", "pi+"): 0.00267,  # PDG 46
    ("anti-K_0*0", "pi0"): 0.0059,  # PDG 59
    ("K-", "pi+", "pi0"): 0.0115,  # PDG 61
    ("anti-K0", "pi0", "pi0"): 0.0091,  # PDG 62
    ("anti-K*0", "pi+", "pi-"): 0.024,  # Belle 2 DecFile Oct 2020
    ("anti-K*0", "pi0", "pi0"): 0.010674,  # Belle 2 DecFile Oct 2020
    ("K-", "rho+", "pi0"): 0.006257,  # Belle 2 DecFile Oct 2020
    ("K-", "pi+", "rho0"): 0.005328,  # Belle 2 DecFile Oct 2020
    ("K-", "pi+", "omega"): 0.031,  # PDG 104 Belle 2 has much smaller value
    ("K-", "pi+", "eta"): 0.009202,  # Belle 2 DecFile Oct 2020
    ("K-", "pi+", "eta'"): 0.00643,  # PDG 106
    ("anti-K*0", "pi+", "pi-", "pi0"): 0.013,  # PDG 86
    ("K-", "pi+", "pi+", "pi-"): 0.0181,  # PDG 80
    ("anti-K0", "pi+", "pi-", "pi0"): 2 * 0.052,  # Checked with PDG 81
    ("K-", "pi+", "pi0", "pi0"): 0.0886,  # PDG 84
    ("anti-K0", "pi0", "pi0", "pi0"): 0.00,  # not found
    ("K-", "pi+", "pi+", "pi-", "pi0"): 0.0049,  # Checked with PDG 85
    ("K-", "pi+", "pi0", "pi0", "pi0"): 0.00,  # not found
    ("anti-K0", "pi+", "pi-", "pi0", "pi0"): 0.026,  # Checked with PDG 97
    ("anti-K0", "pi+", "pi-", "pi0", "pi0", "pi0"): 0.0268,  # Checked
    ("phi", "anti-K0"): 0.00203,  # PDG 114
    ("anti-K0", "K+", "K-"): 2 * (0.00442 - 0.00203),  # PDG (109 - 114) via K_S0
    ("K+", "K-"): 0.00408,  # PDG 185
    ("K*+", "K-"): 0.00189 + 0.00062,  # PDG 189 + 200
    ("K0", "K-", "pi+"): 2 * 0.0033,  # PDG 187 via K_S0
    ("anti-K0", "K+", "pi-"): 2 * 0.00217,  # PDG 198 via K_S0
    ("K+", "K-", "pi0", "pi0"): 0.002761,  # Belle 2 DecFile Oct 2020
    ("anti-K0", "K0", "pi+", "pi-"): 4 * 0.00122,  # PDG 244 via K_S0
    ("K+", "K-", "anti-K0", "pi0"): 0.00,  # not found
    ("K+", "K-", "pi+", "pi-", "pi0"): 0.0031,  # PDG 246
    ("rho+", "pi-"): 0.0101,  # PDG 126
    ("rho-", "pi+"): 0.00515,  # PDG 128
    ("pi+", "pi+", "pi-", "pi-"): 0.00756,  # PDG 143
    ("pi+", "pi-", "pi0", "pi0"): 0.0102,  # PDG 170
    ("pi+", "pi-", "pi+", "pi-", "pi0"): 0.0042,  # PDG 174
    ("pi+", "pi-", "pi0", "pi0", "pi0"): 0.00,  # not found
}  # type: BFDictType

# define the weight dicts
d_charged_bf_weight_dict = generate_bf_weight_dict(
    belle_bf_dict=belle_d_charged_bf_dict,
    updated_bf_dict=updated_d_charged_bf_dict,
)

anti_d_charged_bf_weight_dict = generate_charge_conjugate_bf_weight_dict(
    bf_weight_dict=d_charged_bf_weight_dict,
)

d_zero_bf_weight_dict = generate_bf_weight_dict(
    belle_bf_dict=belle_d_zero_bf_dict,
    updated_bf_dict=updated_d_zero_bf_dict,
)

anti_d_special_weights = {
    (213, -211): updated_d_zero_bf_dict[("rho-", "pi+")] / belle_d_zero_bf_dict[(213, -211)],  # rho+ pi-
    (-213, 211): updated_d_zero_bf_dict[("rho+", "pi-")] / belle_d_zero_bf_dict[(-213, 211)],  # rho- pi+
}  # type: PDGBFDictType

anti_d_additional_weights = {
    (-321, 321, 321, -211): 1  # add this because of messed up Belle Decay File, where decay is defined twice
}  # type: PDGBFDictType

anti_d_zero_bf_weight_dict = generate_charge_conjugate_bf_weight_dict(
    bf_weight_dict=d_zero_bf_weight_dict,
    special_weights=anti_d_special_weights,
    additional_weights=anti_d_additional_weights,
)

if __name__ == "__main__":
    print(sum([v.bf for v in belle_d_charged_bf]))
