"""
Fit observable infos for fit setups of dedicated B -> D** l nu BF or/and efficiency calibration fit.

Felix Metzner 2023
"""


from typing import Tuple

from rdstar.utilities.logging import log_to_default

from rdstar.offline_analysis.histogram_variables import RDStarHistVars

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_info_container import FitObservableInfo


__all__ = [
    "DssCalObsSetup1",
    "DssCalObsSetup2",
    "DssCalObsSetup3",
]


logger = log_to_default(name=__name__)


class DssCalObsSetup1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_dstarstar_cal_fit_cut_range,
        alt_scope=None,
        alt_binning=None,
    )

    dss_cal_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    dss_obs_info_B_to_DLNeutralPi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=24),)
    dss_obs_info_B_to_DStarLNeutralPi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=24),)
    dss_obs_info_B_to_DLChargedPi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=24),)
    dss_obs_info_B_to_DStarLChargedPi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=24),)


class DssCalObsSetup2:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.diff_dstar_d_mass_for_dss_cal_with_lt0p3_cut,
        alt_scope=None,
        alt_binning=None,
    )

    dss_cal_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    dss_obs_info_B_to_DLNeutralPi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=21),)
    dss_obs_info_B_to_DStarLNeutralPi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=21),)
    dss_obs_info_B_to_DLChargedPi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=21),)
    dss_obs_info_B_to_DStarLChargedPi: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=21),)


class DssCalObsSetup3:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.missing_mass_dstarstar_cal_fit_cut_range,
        alt_scope=None,
        alt_binning=None,
    )
    obs2 = FitObservableInfo(
        hist_var=RDStarHistVars.diff_dstar_d_mass_for_dss_cal_with_lt0p3_cut,
        alt_scope=None,
        alt_binning=None,
    )

    dss_cal_obs_infos: Tuple[FitObservableInfo, ...] = (obs1, obs2)

    dss_obs_info_B_to_DLNeutralPi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=8),
        obs2.reset(scope=None, binning=7),
    )
    dss_obs_info_B_to_DStarLNeutralPi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=3),
        obs2.reset(scope=None, binning=7),
    )
    dss_obs_info_B_to_DLChargedPi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=8),
        obs2.reset(scope=None, binning=7),
    )
    dss_obs_info_B_to_DStarLChargedPi: Tuple[FitObservableInfo, ...] = (
        obs1.reset(scope=None, binning=8),
        obs2.reset(scope=None, binning=7),
    )
